/*-------------------------------------------------------------------------+
| Copyright 2016 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.CompositePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.opt4j.core.Genotype;
import org.opt4j.core.genotype.CompositeGenotype;

/**
 * Allows to capture the already created/decoded {@link Phenotype}s and pass them to their successor
 * creators/decoders as a flat map that associates a {@link Phenotype}s type with its instance.
 * Using a {@link CompositePhenotype} for this job allows to "protect" the internal map from
 * unintended read/write operations.
 * 
 * @author diewald
 */
public class FlatGenotypeMap extends CompositeGenotype<Class<? extends Genotype>, Genotype> {
	// No "real" implementation required. The base class does this job.

	/** {@inheritDoc} */
	@Override
	public String toString() {
		String ownName = getClass().getSimpleName() + ": \n";
		StringBuilder strBuilder = new StringBuilder(ownName);
		map.keySet().forEach(g -> strBuilder.append("-> " + g.getSimpleName() + "\n"));
		return strBuilder.toString();
	}
}
