/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype;

import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.opt4j.core.Genotype;
import org.opt4j.core.genotype.CompositeGenotype;

/**
 * Interface for {@link CompositePhenotype}s generated by decoders which operate on
 * {@link CompositeGenotype}s.
 * 
 * @author diewald
 */
public interface CompositePhenotype<G extends Genotype>
		extends
		Phenotype,
		Iterable<Entry<Class<? extends ComposablePhenotype<? extends Genotype>>, ComposablePhenotype<? extends Genotype>>> {

	/**
	 * Returns the {@link ComposablePhenotype} that corresponds to the given
	 * {@link ComposablePhenotype} class.
	 */
	<GT extends Genotype, CP extends ComposablePhenotype<GT>> CP getSubPhenotype(
			Class<CP> subPhenotype);

	/**
	 * Defines a correspondence between a {@link ComposableGenotype} class and its
	 * {@link ComposablePhenotype}.
	 */
	<GT extends Genotype, CP extends ComposablePhenotype<GT>> void putSubPhenotype(
			Class<CP> subGenotype, CP curPhenotype);

	/**
	 * Returns the collection of {@link ComposablePhenotype}s that make up this
	 * {@link ComposableGenotype}.
	 */
	List<ComposablePhenotype<? extends Genotype>> getSubPhenotypes();

	/** Returns the Class -> {@link Phenotype} entries of the enclosed map. */
	Set<Class<? extends ComposablePhenotype<? extends Genotype>>> keySet();
}
