/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import org.opt4j.core.Genotype;

/**
 * Base class for {@link CompositePhenotype}s which consists of {@link ComposablePhenotype}s.
 * 
 * @author diewald
 */
public abstract class CompositePhenotypeBase<G extends Genotype> implements CompositePhenotype<G> {

	/**
	 * Map which correlates the {@link ComposablePhenotype} classes with their corresponding
	 * {@link ComposablePhenotype}s, out of which this {@link CompositePhenotype} consists.
	 */
	protected LinkedHashMap<Class<? extends ComposablePhenotype<? extends Genotype>>, ComposablePhenotype<? extends Genotype>> subPhenotypes =
			new LinkedHashMap<>();

	/** {@inheritDoc} */
	@Override
	public <GT extends Genotype, CP extends ComposablePhenotype<GT>> CP getSubPhenotype(
			Class<CP> subGenotypeClass) {
		for(Entry<Class<? extends ComposablePhenotype<? extends Genotype>>, ComposablePhenotype<? extends Genotype>> composablePheno : subPhenotypes
				.entrySet()) {
			if(subGenotypeClass.isAssignableFrom(composablePheno.getKey())) {
				@SuppressWarnings("unchecked") CP retPhenotype = (CP)composablePheno.getValue();
				return retPhenotype;
			}
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public <GT extends Genotype, CP extends ComposablePhenotype<GT>> void putSubPhenotype(
			Class<CP> subGenotypeClass, CP subPhenotype) {
		subPhenotypes.put(subGenotypeClass, subPhenotype);
	}

	/** {@inheritDoc} */
	@Override
	public List<ComposablePhenotype<?>> getSubPhenotypes() {
		ArrayList<ComposablePhenotype<?>> retList = new ArrayList<>();
		subPhenotypes.values().stream().forEachOrdered(p -> retList.add(p));
		return retList;
	}

	/** {@inheritDoc} */
	@Override
	public
			Iterator<Entry<Class<? extends ComposablePhenotype<? extends Genotype>>, ComposablePhenotype<? extends Genotype>>>
			iterator() {
		return subPhenotypes.entrySet().iterator();
	}

	/** {@inheritDoc} */
	@Override
	public Set<Class<? extends ComposablePhenotype<? extends Genotype>>> keySet() {
		return subPhenotypes.keySet();
	}
}
