/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.problem;

import java.util.List;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create.ComposableCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create.CompositeCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.CompositeDecoder;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ComposableGenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.CompositePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.ProblemModuleBase;
import org.opt4j.core.Genotype;
import org.opt4j.core.genotype.CompositeGenotype;
import org.opt4j.core.problem.Creator;
import org.opt4j.core.problem.Decoder;
import org.opt4j.core.problem.ProblemModule;

/**
 * Base class for {@link ProblemModule}s which are described by sub {@link ProblemModule}s. Used for
 * optimization problems that are described by a {@link ComposableGenotype}.
 * 
 * @author diewald
 */
public abstract class CompositeProblemModuleBase<C extends CompositeGenotype<Class<? extends ComposableGenotype>, ComposableGenotype>, P extends CompositePhenotype<?>, CD extends CompositeDecoder<C, P>>
		extends ProblemModuleBase<C, CompositeCreator<C>, CD> {

	/**
	 * Helper method to register sub problems at the composite problem description (module).
	 * 
	 * @param subGenotypeClass
	 *            {@link Genotype} (class) encoding the problem description.
	 * @param subCreator
	 *            {@link Creator} for initial encodings of the particular sub problem (starting
	 *            points).
	 * @param subDecoder
	 *            {@link Decoder} to derive sub-solutions from the sub encodings. If the given
	 *            {@code subDecoder} equals {@code null} no {@link Decoder} will be registered for
	 *            the given {@subGenotype}.
	 * @throws IllegalArgumentException
	 */
	// TODO: document exception.
	public
			<CG extends ComposableGenotype, CP extends ComposablePhenotype<CG>, CC extends ComposableCreator<CG>, D extends ComposableDecoder<CG, CP>>
			void registerSubProblem(Class<CG> subGenotypeClass, CC subCreator, D subDecoder)
					throws IllegalArgumentException {
		creator.registerSubCreator(subCreator);
		if(subDecoder != null) {
			decoder.registerSubDecoder(subGenotypeClass, subDecoder);
		}
	}

	/**
	 * Helper method to register sub problems at the composite problem description (module).
	 * 
	 * @param subGenotypeClass
	 *            Genotype (class) encoding the problem description.
	 * @param subCreator
	 *            Creator for initial encodings of the particular sub problem (starting points).
	 * @param subDecoders
	 *            Collection of {@link Decoder}s to derive sub-solutions from the sub encodings.
	 * @throws IllegalArgumentException
	 */
	// TODO: document exception.
	public
			<CG extends ComposableGenotype, CP extends ComposablePhenotype<CG>, CC extends ComposableCreator<CG>, D extends ComposableDecoder<CG, CP>>
			void registerSubProblem(Class<CG> subGenotypeClass, CC subCreator, List<D> subDecoders)
					throws IllegalArgumentException {
		creator.registerSubCreator(subCreator);
		if(subDecoders != null) {
			for(D currDecoder : subDecoders) {
				decoder.registerSubDecoder(subGenotypeClass, currDecoder);
			}
		}
	}
}
