/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection;

import org.opt4j.core.optimizer.Operator;
import org.opt4j.operators.AbstractGenericOperator;

/**
 * Interface to define custom operator selection strategies. In order apply such strategy in the
 * optimization run of Opt4J, for each operator type (e.g., mutation, crossover, ...) one instance
 * of a class implementing this interface is shared between dedicated per-operator
 * instances of the {@link OperatorSelectorPredicate} wrapper.
 * 
 * @author barner
 */
public interface IOperatorSelector {

	/**
	 * Returns true iff the operator managed by the corresponding {@link OperatorSelectorPredicate}
	 * should be selected at the current stage of the optimization run.
	 * 
	 * {@link OperatorSelectorPredicate#evaluate(org.opt4j.core.Genotype)} (which is triggered by
	 * {@link AbstractGenericOperator#getOperator(org.opt4j.core.Genotype)}) delegates the
	 * actual work to this method
	 */
	public abstract boolean evaluate(Operator<?> operator);

	/**
	 * Registers an operator with this {@link IOperatorSelector}. This method is
	 * invoked from the constructor
	 * {@link OperatorSelectorPredicate#OperatorSelectorPredicate(Operator, IOperatorSelector)}.
	 */
	public abstract void register(Operator<?> operator);
}
