/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.partitionmapping;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.platform.PlatformCommunicationGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoderBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.ArchExpSubDecoder;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ICommunicationResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.tooling.common.util.LambdaUtils;
import org.jgrapht.Graphs;
import org.jgrapht.graph.DefaultEdge;

/**
 * 
 * @author diewald
 */
public class PartitionMappingDecoderGraph extends
		ComposableDecoderBase<PlatformCommunicationGraphEncoding, PlatformCommunicationGraphEncoding>
		implements ArchExpSubDecoder {
	/** {@inheritDoc} */
	@Override
	public PlatformCommunicationGraphEncoding decodeComposable(
			PlatformCommunicationGraphEncoding genotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		PartitionMappingEncoding pmEnc =
				decodedPhenotypes.getEncoding(PartitionMappingEncoding.class);
		assert (pmEnc != null) : "The required platform communication graph encoding could not" +
				" be found in the map of already decoded phenotypes.";

		for(IResourceAdapter<?> execUnit : genotype.getActualGraph().vertexSet()) {
			for(IResourceAdapter<?> partition : pmEnc.getRequesters(execUnit)) {
				genotype.getActualGraph().addVertex(partition);
				genotype.getActualGraph().addEdge(partition, execUnit);
				genotype.getActualGraph().addEdge(execUnit, partition);
			}
		}

		return genotype;
	}

	/** {@inheritDoc} */
	@Override
	public PlatformCommunicationGraphEncoding updateComposable(
			PlatformCommunicationGraphEncoding inGenotype,
			PlatformCommunicationGraphEncoding outGenotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		PartitionMappingEncoding pmEnc =
				decodedPhenotypes.getEncoding(PartitionMappingEncoding.class);
		assert (pmEnc != null) : "The required partition mapping encoding could not" +
				" be found in the map of already decoded phenotypes.";
		// Remove non-existing Partitions (from a previous iteration).
		Collection<IResourceAdapter<?>> removeVertices = new ArrayList<>();
		for(IResourceAdapter<?> resource : inGenotype.getActualGraph().vertexSet()) {
			if((resource instanceof Partition) && !pmEnc.getRequesters().contains(resource)) {
				Collection<DefaultEdge> edgeSet = inGenotype.getActualGraph().edgesOf(resource);
				inGenotype.getActualGraph().removeAllEdges(edgeSet);
				removeVertices.add(resource);
			}
		}
		inGenotype.getActualGraph().removeAllVertices(removeVertices);
		// Add missing Partitions.
		for(Partition partition : pmEnc.getRequesters()) {
			if(!inGenotype.getActualGraph().containsVertex(partition)) {
				// We have only one Partition per entry --> Multiples for task mappings.
				// TODO: Correct the return type for single and multi returns.
				IResourceAdapter<?> execUnit =
						pmEnc.getAllocatedResourcesFor(partition).iterator().next();
				inGenotype.getActualGraph().addVertex(partition);
				// FIXME: HACK!!! Connects the Partitions to the communication resource attached to
				// the phys exec unit. This is used to avoid larger reworks in the TDMA Message
				// scheduler. The scheduler must be reconsidered to reflect internal comm. of a
				// core/processor correctly.
				Collection<IResourceAdapter<?>> commResources =
						Graphs.neighborListOf(inGenotype.getActualGraph(), execUnit);
				IResourceAdapter<?> commresource = LambdaUtils
						.getFirst(commResources, r -> r instanceof ICommunicationResourceAdapter)
						.get();
				inGenotype.getActualGraph().addEdge(partition, commresource);
				inGenotype.getActualGraph().addEdge(commresource, partition);
			}
		}
		return inGenotype;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> reqTypes = new ArrayList<>();
		reqTypes.add(PlatformCommunicationGraphEncoding.class);
		reqTypes.add(PartitionMappingEncoding.class);
		return reqTypes;
	}
}
