/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.instantiate;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.TaskMappingDecoderBase;

/**
 * 
 * @author diewald
 */
public class TaskInstanceResourceAlignmentDecoder extends
		TaskMappingDecoderBase<InstantiatedTaskMappingEntry, InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry>, InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry>> {

	/** {@inheritDoc} */
	@Override
	public InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry> decodeComposable(
			InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry> genotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		PartitionMappingEncoding pmEncoding =
				decodedPhenotypes.getEncoding(PartitionMappingEncoding.class);
		assert (pmEncoding != null);
		genotype.updatePartitionMapping(pmEncoding);
		return genotype;
	}

	/** {@inheritDoc} */
	@Override
	public InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry> updateComposable(
			InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry> inGenotype,
			InstantiatedTaskMappingEncoding<InstantiatedTaskMappingEntry> outGenotype,
			FlatPhenotypeMap<Phenotype> decodedPhenotypes) {
		PartitionMappingEncoding pmEncoding =
				decodedPhenotypes.getEncoding(PartitionMappingEncoding.class);
		assert (pmEncoding != null);
		inGenotype.updatePartitionMapping(pmEncoding);
		return alignWithResourceModifications(inGenotype, decodedPhenotypes);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> reqTypes = new ArrayList<>();
		reqTypes.add(InstantiatedTaskMappingEncoding.class);
		reqTypes.add(PartitionMappingEncoding.class);
		return reqTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		Collection<Class<?>> optTypes = new ArrayList<>();
		return optTypes;
	}
}
